<?php

/**
 * @file Basic library of functions
 */

/**
 * Alternative to array_merge_recursive which produces cleaner arrays.
 *
 * @param $array
 *  Array we are overriding with new values.
 * @param $override
 *  Array to be merged into $array. Any values in $override will recursively
 *  replace those values for keys in $array.
 *
 * @return
 *  Array with new values merged in.
 *
 * @see http://us3.php.net/array_merge_recursive
 */
function dt_array_merge(&$array, $override) {
  if (is_array($array)) {
    foreach ($override as $key => $info) {
      if (!isset($array[$key])) {
        $array[$key] = $info;
      }
      elseif (is_array($override[$key]) && is_array($array[$key])) {
        dt_array_merge($array[$key], $override[$key]);
      }
      else {
        $array[$key] = $info;
      }
    }
  }
}

/**
 * Build an array of a specified key value in sub-arrays.
 *
 * @param $array
 *  An array. This function only affects elements that are arrays or objects.
 * @param $key
 *  (optional) key value to check in sub-arrays. Defaults to 'value'.
 *
 * @return
 *  Array of key => value pairs, where the key refers to the sub-array, and
 *  value is the specified element from the sub-array. If searching for the
 *  '#default_value' in a $form array, you would get:
 *  array('form_element1' => 'first_value', 'form_element2' => 'second_value')
 */
function dt_array_gather($array, $key = 'value') {
  $vals = array();
  if (is_array($array)) {
    foreach ($array as $k => $v) {
      $v = is_object($v) ? array($v) : $v;
      if (isset($v[$key])) {
        $vals[$k] = $v[$key];
      }
    }
  }
  return $vals;
}

/**
 * Convert array to a comma-separated list with an optional and/or at the end.
 *
 * @param $array
 *  Array of items for conversation to string.
 * @param $op
 *  (optional) String that connects the last element of the array to the end of
 *  the string. Defaults to 'and'.
 *
 * @return String
 *  - Multiple elements: "apple, banana, orange, and pear"
 *  - Single element: "apple"
 */
function dt_grammar_list($array, $op = NULL) {
  if (empty($op)) {
    $op = t('and');
  }
  $last = array_pop($array);
  return (!empty($array) ? implode(', ', $array) . ' ' . $op . ' ' : '') . $last;
}

/**
 * Sort an array by weight key.
 *
 * @see element_sort
 */
function dt_element_sort($a, $b) {
  $a_weight = (is_array($a) && isset($a['weight'])) ? $a['weight'] : 0;
  $b_weight = (is_array($b) && isset($b['weight'])) ? $b['weight'] : 0;
  if ($a_weight == $b_weight) {
    return 0;
  }
  return ($a_weight < $b_weight) ? -1 : 1;
}

/**
 * Toggle off capture of messages for display on page.
 *
 * drupal_set_message places messages on the page. You do not always want the
 * messages that would be generated by an operation, this function allows you
 * to discard any messages generated during such operations.
 *
 * When you turn on the "denied" state, the current messages are statically
 * cached. When you turn "denied" off, it restores the saved messages over any
 * that Drupal collected in the interim.
 *
 * @param Boolean $denied
 *   (optional) When TRUE, no messages reported by drupal_set_message will be
 *   saved for use. When FALSE, new messages will be saved. Defaults to TRUE.
 *
 * @see drupal_set_message
 */
function dt_drupal_messages_denied($denied = TRUE) {
  static $session_copy;
  if ($start) {
    $session_copy = isset($_SESSION['messages']) ? $_SESSION['messages'] : array();
  }
  elseif (isset($session_copy)) {
    if (!empty($_SESSION['messages'])) {
      foreach ($_SESSION['messages'] as $key => $message_array) {
        foreach ($message_array as $pos => $message) {
          if (!isset($session_copy[$key][$pos])) {
            unset($_SESSION['messages'][$key][$pos]);
          }
        }
        if (empty($_SESSION['messages'][$key])) {
          unset($_SESSION['messages'][$key]);
        }
      }
    }
  }
}

/**
 * Set a path for use by drupal_goto without interrupting the page load.
 *
 * @see drupal_goto, http://drupal.org/node/732542
 */
function dt_drupal_goto_delay($in_path = NULL) {
  static $path;
  if ($in_path) {
    $path = $in_path;
  }
  return $path;
}

/**
 * Wrapper to include files.
 *
 * module_load_include() makes some odd assumptions about file name structure,
 * by implementing our own wrapper, we avoid that confusion.
 *
 * @param $extension
 *  Name of the type of extension the file is a part of.
 * @param $file
 *  File name. May be a path relative to the location of the extension.
 * @param $type
 *  (optional) Type of extension. Defaults to 'module'. May include module,
 *   theme, theme_engine, or profile.
 * @param $once
 *  (option) Whether to use include or include_once. Defaults to include_once.
 *
 * @see module_load_include
 */
function dt_load_include($extension, $file, $type = 'module', $once = TRUE) {
  // if ($type == 'library') {
  //  $path = libraries_get_path($extension) . '/' . $file;
  //}
  //else {
  $path = drupal_get_path($type, $extension) . '/' . $file;
  //}
  if ($once) {
    include_once($path);
  }
  else {
    include($path);
  }
}
